@@ -32,9 +32,7 @@
 #include <boost/filesystem/path.hpp>
 #include <boost/algorithm/string/find.hpp>
 
-#include <libxml++/parsers/textreader.h>
-#include <libxml++/parsers/domparser.h>
-#include <libxml++/nodes/textnode.h>
+#include <libxml/parser.h>
 
 #include <glibmm/i18n.h>
 #include <gtkmm/button.h>
@@ -51,7 +49,9 @@
 #include "sharp/files.hpp"
 #include "sharp/map.hpp"
 #include "sharp/string.hpp"
+#include "sharp/xml.hpp"
 #include "sharp/xmlconvert.hpp"
+#include "sharp/xmlreader.hpp"
 #include "sharp/xmlwriter.hpp"
 
 
@@ -394,7 +394,7 @@ namespace gnote {
   Note::Note(NoteData * _data, const std::string & filepath, NoteManager & _manager)
     : m_data(_data)
     , m_filepath(filepath)
-    , m_save_needed(true)
+    , m_save_needed(false)
     , m_is_deleting(false)
     , m_manager(_manager)
     , m_window(NULL)
@@ -650,7 +650,7 @@ namespace gnote {
   void Note::add_tag(const Tag::Ptr & tag)
   {
     if(!tag) {
-      throw sharp::Exception ("Note.AddTag () called with a null tag.");
+      throw sharp::Exception ("note::add_tag() called with a NULL tag.");
     }
     tag->add_note (*this);
 
@@ -809,29 +809,24 @@ namespace gnote {
     // were to throw an XmlException in the middle of processing,
     // a note could be damaged.  Therefore, we check for parseability
     // ahead of time, and throw early.
-    {
-      xmlpp::DomParser parser;
-      parser.parse_memory(foreignNoteXml);
-      if(!parser) {
-        throw sharp::Exception("invalid XML in foreignNoteXml");
-      }
-    }
+    xmlDocPtr doc = xmlParseDoc((const xmlChar *)foreignNoteXml.c_str());
 
-    xmlpp::TextReader xml((const unsigned char*)foreignNoteXml.c_str(), 
-                          foreignNoteXml.size());
+    if(!doc) {
+      throw sharp::Exception("invalid XML in foreignNoteXml");
+    }
+    xmlFreeDoc(doc);
+
+    sharp::XmlReader xml;
+    xml.load_buffer(foreignNoteXml);
 
     // Remove tags now, since a note with no tags has
     // no "tags" element in the XML
-    std::list<Tag::Ptr> tag_list;
-    for(std::list<Tag::Ptr>::const_iterator iter = tag_list.begin();
-        iter != tag_list.end(); ++iter) {
-      remove_tag(*iter);
-    }
-    Glib::ustring name;
+    std::list<Tag::Ptr> new_tags;
+    std::string name;
 
     while (xml.read()) {
       switch (xml.get_node_type()) {
-      case xmlpp::TextReader::Element:
+      case XML_READER_TYPE_ELEMENT:
         name = xml.get_name();
         if (name == "title") {
           set_title(xml.read_string());
@@ -852,17 +847,16 @@ namespace gnote {
             sharp::XmlConvert::to_date_time(xml.read_string ());
         }
         else if(name == "tags") {
-          xmlpp::DomParser parser;
-          parser.parse_memory(xml.read_outer_xml());
-          if(parser) {
-            const xmlpp::Document * doc2 = parser.get_document();
+          xmlDocPtr doc2 = xmlParseDoc((const xmlChar*)xml.read_outer_xml().c_str());
+          if(doc2) {
             std::list<std::string> tag_strings;
-            parse_tags (doc2->get_root_node(), tag_strings);
+            parse_tags (doc2->children, tag_strings);
             for(std::list<std::string>::const_iterator iter = tag_strings.begin();
                 iter != tag_strings.end(); ++iter) {
               Tag::Ptr tag = TagManager::obj().get_or_create_tag(*iter);
-              add_tag(tag);
+              new_tags.push_back(tag);
             }
+            xmlFreeDoc(doc2);
           }
           else {
             DBG_OUT("loading tag subtree failed");
@@ -881,27 +875,42 @@ namespace gnote {
 
     xml.close ();
 
+    std::list<Tag::Ptr> tag_list;
+    get_tags(tag_list);
+    
+    for(std::list<Tag::Ptr>::const_iterator iter = tag_list.begin();
+        iter != tag_list.end(); ++iter) {
+      if(find(new_tags.begin(), new_tags.end(), *iter) == new_tags.end()) {
+        remove_tag(*iter);
+      }
+    }
+    for(std::list<Tag::Ptr>::const_iterator iter = new_tags.begin();
+        iter != new_tags.end(); ++iter) {
+      add_tag(*iter);
+    }
+    
     // Allow method caller to specify ChangeType (mostly needed by sync)
     queue_save (changeType);
   }
 
 
-  void Note::parse_tags(const xmlpp::Node *tagnodes, std::list<std::string> & tags)
+  void Note::parse_tags(const xmlNodePtr tagnodes, std::list<std::string> & tags)
   {
-    xmlpp::NodeSet nodes = tagnodes->find("//*");
-    for(xmlpp::NodeSet::const_iterator iter = nodes.begin();
+    sharp::XmlNodeSet nodes = sharp::xml_node_xpath_find(tagnodes, "//*");
+    
+    if(nodes.empty()) {
+      return;
+    }
+    for(sharp::XmlNodeSet::const_iterator iter = nodes.begin();
         iter != nodes.end(); ++iter) {
-      const xmlpp::Node * node = *iter;
-      if(node->get_name() != "tag") {
-        continue;
-      }
-      const xmlpp::Element * content = dynamic_cast<const xmlpp::Element*>(node);
-      if(content) {
-        const xmlpp::TextNode * textnode = content->get_child_text();
-        if(textnode) {
-          std::string tag = textnode->get_content();
-          DBG_OUT("found tag %s", tag.c_str());
-          tags.push_back(tag);
+
+      const xmlNodePtr node = *iter;
+      if(xmlStrEqual(node->name, (const xmlChar*)"tag") && (node->type == XML_ELEMENT_NODE)) {
+        xmlChar * content = xmlNodeGetContent(node);
+        if(content) {
+          DBG_OUT("found tag %s", content);
+          tags.push_back((const char*)content);
+          xmlFree(content);
         }
       }
     }
@@ -956,7 +965,7 @@ namespace gnote {
       // NOTE: Sharing the same TagTable means
       // that formatting is duplicated between
       // buffers.
-      m_tag_table = NoteTagTable::Ptr(&NoteTagTable::instance());
+      m_tag_table = NoteTagTable::instance();
     }
     return m_tag_table;
   }
@@ -965,7 +974,7 @@ namespace gnote {
   {
     if(!m_buffer) {
       DBG_OUT("Creating buffer for %s", m_data.data().title().c_str());
-      m_buffer = Glib::RefPtr<NoteBuffer>(new NoteBuffer(get_tag_table(), *this));
+      m_buffer = NoteBuffer::create(get_tag_table(), *this);
       m_data.set_buffer(m_buffer);
 
       m_buffer->signal_changed().connect(
@@ -1088,13 +1097,13 @@ namespace gnote {
     NoteData *note = new NoteData(uri);
     std::string version;
 
-    xmlpp::TextReader xml(read_file);
+    sharp::XmlReader xml(read_file);
 
-    Glib::ustring name;
+    std::string name;
 
     while (xml.read ()) {
       switch (xml.get_node_type()) {
-      case xmlpp::TextReader::Element:
+      case XML_READER_TYPE_ELEMENT:
         name = xml.get_name();
         
         if(name == "note") {
@@ -1136,17 +1145,17 @@ namespace gnote {
           note->y() = boost::lexical_cast<int>(xml.read_string());
         }
         else if(name == "tags") {
-          xmlpp::DomParser parser;
-          parser.parse_memory(xml.read_outer_xml());
-          if(parser) {
-            const xmlpp::Document * doc2 = parser.get_document();
+          xmlDocPtr doc2 = xmlParseDoc((const xmlChar*)xml.read_outer_xml().c_str());
+
+          if(doc2) {
             std::list<std::string> tag_strings;
-            Note::parse_tags(doc2->get_root_node(), tag_strings);
+            Note::parse_tags(doc2->children, tag_strings);
             for(std::list<std::string>::const_iterator iter = tag_strings.begin();
                 iter != tag_strings.end(); ++iter) {
               Tag::Ptr tag = TagManager::obj().get_or_create_tag(*iter);
               note->tags()[tag->normalized_name()] = tag;
             }
+            xmlFreeDoc(doc2);
           }
           else {
             DBG_OUT("loading tag subtree failed");
@@ -1166,7 +1175,7 @@ namespace gnote {
     if (version != NoteArchiver::CURRENT_VERSION) {
       // Note has old format, so rewrite it.  No need
       // to reread, since we are not adding anything.
-      DBG_OUT("Updating note XML to newest format...");
+      DBG_OUT("Updating note XML from %s to newest format...", version.c_str());
       NoteArchiver::write (read_file, *note);
     }
     return note;
@@ -1333,11 +1342,13 @@ namespace gnote {
   std::string NoteArchiver::get_title_from_note_xml(const std::string & noteXml) const
   {
     if (!noteXml.empty()) {
-      xmlpp::TextReader xml((const unsigned char*)noteXml.c_str(), noteXml.size());
+      sharp::XmlReader xml;
+
+      xml.load_buffer(noteXml);
 
       while (xml.read ()) {
         switch (xml.get_node_type()) {
-        case xmlpp::TextReader::Element:
+        case XML_READER_TYPE_ELEMENT:
           if (xml.get_name() == "title") {
             return xml.read_string ();
           }
