@@ -1,6 +1,7 @@
 /*
  * gnote
  *
+ * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -34,10 +35,11 @@
 #include <glibmm/spawn.h>
 #include <gtkmm/icontheme.h>
 #include <gtkmm/image.h>
+#include <gtkmm/label.h>
 #include <gtkmm/stock.h>
 #include <gtkmm/textbuffer.h>
-#include <libxml++/parsers/textreader.h>
 
+#include "sharp/xmlreader.hpp"
 #include "sharp/xmlwriter.hpp"
 #include "sharp/string.hpp"
 #include "sharp/uri.hpp"
@@ -88,18 +90,13 @@ namespace gnote {
       }
     }
 
+
     void popup_menu(Gtk::Menu &menu, const GdkEventButton * ev)
-    {
-      popup_menu(menu, ev, boost::bind(&get_menu_position, &menu, _1, _2, _3));
-    }
-
-
-    void popup_menu(Gtk::Menu &menu, const GdkEventButton * ev, 
-                    Gtk::Menu::SlotPositionCalc calc)
     {
       menu.signal_deactivate().connect(sigc::bind(&deactivate_menu, &menu));
-      menu.popup(calc, (ev ? ev->button : 0), 
-                  (ev ? ev->time : gtk_get_current_event_time()));
+      menu.popup(boost::bind(&get_menu_position, &menu, _1, _2, _3), 
+                 (ev ? ev->button : 0), 
+                 (ev ? ev->time : gtk_get_current_event_time()));
       if(menu.get_attach_widget()) {
         menu.get_attach_widget()->set_state(Gtk::STATE_SELECTED);
       }
@@ -462,17 +459,17 @@ namespace gnote {
 
         const std::string & i(*iter);
 
-        if(sharp::string_starts_with(i, "#")) {
+        if(Glib::str_has_prefix(i, "#")) {
           continue;
         }
 
         std::string s = i;
-        if(sharp::string_ends_with(i, "\r")) {
+        if(Glib::str_has_suffix(i, "\r")) {
           s.erase(s.end() - 1, s.end());
         }
 
         // Handle evo's broken file urls
-        if(sharp::string_starts_with(s, "file:////")) {
+        if(Glib::str_has_prefix(s, "file:////")) {
           s = sharp::string_replace_first(s, "file:////", "file:///");
         }
         DBG_OUT("uri = %s", s.c_str());
@@ -533,12 +530,13 @@ namespace gnote {
       // this will do for now.
       std::string builder;
 
-      xmlpp::TextReader xml((const xmlChar *)source.c_str(), source.size());
+      sharp::XmlReader xml;
+      xml.load_buffer(source);
 
       while (xml.read ()) {
         switch (xml.get_node_type()) {
-        case xmlpp::TextReader::Text:
-        case xmlpp::TextReader::Whitespace:
+        case XML_READER_TYPE_TEXT:
+        case XML_READER_TYPE_WHITESPACE:
           builder += xml.get_value();
           break;
         default:
@@ -697,25 +695,27 @@ namespace gnote {
     ToolMenuButton::ToolMenuButton(Gtk::Toolbar& toolbar, const Gtk::BuiltinStockID& stock_image, 
                                    const Glib::ustring & label, 
                                    Gtk::Menu * menu)
-      : Gtk::ToggleToolButton(label)
+      : Gtk::ToggleToolButton()
       ,  m_menu(menu)
     {
-      _common_init(*manage(new Gtk::Image(stock_image, toolbar.get_icon_size())));
+      _common_init(*manage(new Gtk::Image(stock_image, toolbar.get_icon_size())),
+                   label);
     }
 
     ToolMenuButton::ToolMenuButton(Gtk::Image& image, 
                                    const Glib::ustring & label, 
                                    Gtk::Menu * menu)
-      : Gtk::ToggleToolButton(label)
+      : Gtk::ToggleToolButton()
       ,  m_menu(menu)
     {
-      _common_init(image);
+      _common_init(image, label);
     }
 
 
-    void ToolMenuButton::_common_init(Gtk::Image& image)
+    void ToolMenuButton::_common_init(Gtk::Image& image, const Glib::ustring & label)
     {
       set_icon_widget(image);
+      set_label_widget(*manage(new Gtk::Label(label, true)));
       property_can_focus() = true;
       gtk_menu_attach_to_widget(m_menu->gobj(), static_cast<Gtk::Widget*>(this)->gobj(),
                                 NULL);
